body {
    margin: 0pt;
    background-color: white;
    font-family: "Verdana", "Tahoma", "Helvetica";
}
div#result {
	border-style: solid;
	border-width: 1px;
	border-color: black;
}
div#header {
	border-style: solid;
	border-width: 0px 0px 1px 0px;
	background-color: rgb(80%,60%,60%);
}
.hidden {
    visibility:hidden;
}
.nowrap {
    white-space: nowrap;
    padding: 0;
    border: 0;
}
.baseline {
    vertical-align: baseline;
}
.title {
	padding: 1pt;
	padding-left: 5pt;
	padding-right: 5pt;
	background-color: white;
}
.big {
    font-size: 24pt;
    font-weight: bold;
    padding-right: 5pt;
}
.menu {
	padding-right: 5pt;
}
.menu2 {
	padding-left: 5pt;
}
.row {
	padding: 1pt;
	padding-left: 5pt;
	white-space: nowrap;
}
.lang {
	width: 75pt;
	display: inline-block;
	text-align: left;
}
.input {
	padding: 1pt;
	padding-left: 5pt;
	background-color: rgb(83%,65%,65%);
}
img {
	border-width: 0px;
/*	border-style: solid; */
}
a:hover img {
	border-color: black;
}
a:visited img {
	border-color: white;
}
table.logos td {
	padding-left: 5px;
	padding-right: 5px;
	text-align: center;
}
tr.text td {	
	padding: 5pt;
}
@font-face {
   font-family: Hung;
   src: url("../fonts/fonts/Hung.ttf") format("truetype");
}
.rovas {
	font-family: Hung;
	width: 100%;
}
table#conv {
	width: 100%;
	border-spacing: 0px;
	border-collapse: collapse;
}
table#conv tr {
	margin: 0px;
}
table#conv td {
	margin: 0px;
	padding: 2pt;
	padding-left: 5pt;
	padding-right: 5pt;
	border-width: 0px;
	border-style: solid;
	white-space: nowrap;	
}

table#conv td.sep {
	border-width: 1px 0px 0px 0px;
	border-color: black;
	border-style: solid;
}

